function dataout = ...
    extract2(startyr, stopyr, z1, z2, data, dc, s1c, s2c, flag)
% EXTRACT2 selects data in startyr - stopyr; can average or subsample
%
% input
%   start = scalar, integer start year
%   stop = scalar, integer stop year
%   z = scalar, add before logarithmic transformation
%   data = array with 3 (or more) columns of nontransformed data, 
%       specified by:
%       dc = scalar, number of column for date, 
%       s1c = scalar, number of colum for Y1,
%       s2c = scalar, number of column for Y2
%   flag = scalar, controls averaging (by months if flag >0.5)
%       and subsampling (if flag < 0)
%       NEW 8 MAY 2016: use flag > 1.5 for arithmetic averaging
% output
%   dataout = array of 4 columns, corresponding in sequence to:
%       YR = vector of year.dec between start and stop 
%       X = vector of log-transformed Y1 variable within year-limits
%       Y = vector of log-transformed Y2 variable within year-limits
%       M = vector of month number for each row of selected data
%
% History and Authorship
% original 'extract', PT, 19-20 March 06
% negative flag results in random choice: PT, 25 Mar 06
% updated PT, 1 Sep 06, to deal better with NaNs
% v2: PT, 8-9 Aug 08, including single array output from 4 vectors
%     19 Aug 08: exclude years with no data when monthly averaging
%     6 Aug 09: replace 'finite' by 'isfinite' for use with Matlab 7.8
%     23 July 2015: updates random number seeding
%      7 Sep 2015: replace z by z1 and z2
%     13 Nov 15: adjust 'backstop' values of z1 and z2
%     8 May 2016: add in-function option for arithmetic averaging  

ASO = 1; % ASO = 1 allows screen output, = 0 suppresses most
% suppress except when checking correct working
if flag > 1.5, 
    AA = 1; % AA = 1 gives arithmetic averaging, if averaging on
else
    % default is logarithmic averaging after log(x+z) transformation
    AA = 0;
end

if ASO > 0.5, 
    fprintf('\n function %s is running to get data subset \n', mfilename );
end
% check dates and z
startyr=floor(startyr); % in case startyr is not an integer
stopyr=0.999 +floor(stopyr); % ditto and to include whole of year
if stopyr<=startyr, stopyr=startyr+1; 
    fprintf('%s\n', 'EX2: year error corrected in extract'); 
end
% 13 Nov 2015: the user might legitimately have supplied
% very small z values; all that's needed is to avoid taking logs
% of zero in transform. 
backstop = realmin('single'); % about 1e-38
if z1 < backstop, z1 = backstop; 
    fprintf(['%s\n', 'z1 set to ' z1 ' by extract2']); 
end
if z2 < backstop, z2 = backstop; 
    fprintf(['%s\n', 'z2 set to ' z2 ' by extract2']); 
end
%
% sort data into date order (just in case it is not)
data2=sortrows(data,dc);
%
% transform data
year=data2(:,dc);
if AA < 0.5, % transform data now
    XO=log10(z1+data2(:,s1c));
    YO=log10(z2+data2(:,s2c)); 
else % work with arithmetic data (and transform later)
    XO=data2(:,s1c);
    YO=data2(:,s2c);
    if ASO > 0.5, fprintf(' EX2: Working with arithmetic data.\n'); end
end
%   
%% EXTRACT VALUES
% this way may be cumbersome but it should work with data in any order!
X=zeros(size(XO));
Y=zeros(size(YO));
YR=zeros(size(year));
M=zeros(size(year));
for N=1:length(year),
	if year(N)>=startyr;
		if year(N)<=stopyr;
			X(N)=XO(N);
			Y(N)=YO(N);
			YR(N)=year(N); 
			M(N)=1+floor(rem(year(N),1)*12);% month within year
		else X(N)=NaN;
			Y(N)=NaN;
			YR(N)=NaN;
			M(N)=NaN;
		end
    else X(N)=NaN;
		Y(N)=NaN;
		YR(N)=NaN;
		M(N)=NaN;
	end
end
% remove the NaNs - 
% this bit modified 1 Sep 06 to work in 2 stages
% first stage, delete all rows that have a NaN in the X column
YR=YR(isfinite(X));
Y=Y(isfinite(X));
M=M(isfinite(X));
X=X(isfinite(X));% must do this last!
% second stage, delete all rows that have a NaN in the Y column
YR=YR(isfinite(Y));
X=X(isfinite(Y));
M=M(isfinite(Y));
Y=Y(isfinite(Y));% must do this last!
%
%% AVERAGING
if flag>0.5
    % do averaging if requested;
    % the following does not require sorted data
    MMrow=ceil(stopyr-startyr);
    MMYR=zeros(MMrow, 12);% for totals and means of year
    MMX=MMYR;% for totals and means of X
    MMY=MMYR;% for totals and means of Y
    MMN=MMYR;% for counts
    for N=1:length(YR),
        yrnm=1+floor(YR(N)-startyr); 
        MMYR(yrnm, M(N))=MMYR(yrnm, M(N)) + YR(N);
        MMX(yrnm, M(N))=MMX(yrnm, M(N)) + X(N);
        MMY(yrnm, M(N))=MMY(yrnm, M(N)) + Y(N);
        MMN(yrnm, M(N))=MMN(yrnm, M(N)) + 1;
    end
    warning off; % to avoid warning when some zeros in MMN
    MMYR=MMYR./MMN;
    MMX=MMX./MMN;
    MMY=MMY./MMN;
    warning on;
    % unpack
    clear YR Y X M;
    N=1;
    for row=1:MMrow,
        for col=1:12,
            YR(N)=MMYR(row, col);
            X(N)=MMX(row, col);
            Y(N)=MMY(row, col);
            M(N)=col;
            N=N+1;
        end
    end
    % delete all rows that have a NaN in the YR column (19 Aug 08)
    % (which might arise when there ae empty years or months)
X=X(isfinite(YR));
Y=Y(isfinite(YR));
M=M(isfinite(YR));
YR=YR(isfinite(YR));% must do this last!
%
end % of averaging
%
%% RANDOM VALUES
if flag <0.0
    % pick values at random if requested
    pp=1/-flag;
    if pp>1, pp=1.0; end% to avoid a possible error
    % randomly initialize the random number generator
    % NB this usageworked fine in Matlab 7.1 but is deprecated in 7.8
    % so the next line should be revised (6 Aug 09)
    % rand('state', sum(100*clock));
    % here is the revision (23 July 2015)
    rng('shuffle'); % as recommended by Matlab
    % rand(size X) is a vector of random numbers corresponding to X
    pr=rand(size(X))<pp; % vector of 1s and 0s to give required selection
    YR=YR(pr);
    X=X(pr);
    Y=Y(pr);
    M=M(pr);
end % of subsampling
%  
%% MAKE OUTPUT ARRAY
dataout(:,1) = YR;
if AA < 0.5, % already log-transformed
    dataout(:,2) = X;
    dataout(:,3) = Y;
else
    dataout(:,2) = log10(z1 + X);
    dataout(:,3) = log10(z2 + Y);
    if ASO > 0.5,
        fprintf(' EX2: Data have now been log-transformed \n\n'); end
end
dataout(:,4) = M;
%
% end function