function [envOXY, envIXY, pl ] = findenv2(X,Y,p,sc)
% finds inner and outer envelopes for a set of data in 2-space
% --- findenv ----
% function [envOXY, envIXY ] = findenv2(X,Y)
% PT 19 and 26 Mar 06 and 14 Aug 08
% input
%   vectors X and Y are a set of cartesian co-ordinates
% output
%   envOXY contains outer envelope (X and Y co-ordinates) in 2 columns
%   envIXY contains inner envelope (X and Y co-ordinates) in 2 columns
% --- findenv2 ---   FOR USE WITH PCI1F  -- optional input p
% PT 6-8 Aug 09 - exclude proportion (1-p) outliers from envelopes
%   (if p not supplied, assumed to be 1.0)
%   p is checked; corrected value is returned as pl
% PT 7 Nov 11 - changes to allow checking of operation
%   Discovered that the code treats all values from the bottom left hand 
%   corner of the cloud in the same way - so they might get included
%   or excluded as a group. This does not matter so far as finding
%   co-ordinates for points in the envelopes is concerned.
%   However, the private function 'trimmedwith' has been modified
% 	so that nearly the correct fraction of points is always excluded
% PT 15 May 2013 - standardizing variation along X and Y axes
%   to make envelope fit better - 20 May, scaling set by switch sc

% supply default switch for scaling
if nargin < 4, sc = 0; end
% supply default proportion or check and correct
if nargin < 3, pl = 1.0;
else
    fe = 0; pl = p;
    if p > 1.0, pl = 1.0; fe = 1; end
    if p < 0.5, pl = 0.5; fe = 2; end
    if fe > 0.5, fprintf('\n %s %6f.2 %s %6f.2 \n', ...
        '===FINDENV2: proportion corrected from', p, ' to ', pl);
    end
end

% calculate centre (from medians) and polar co-ordinates around centre
% 15 May 2013 - scaling of X and Y axes
	medX=median(X);% x-axis variable
    % rX=max(X)-min(X);
    rX = std(X);
	medY=median(Y);% y-axis variable
    % rY=max(Y)-min(Y);
    rY = std(Y);
    if sc == 1, % scale axes
        [angle, vector]=cart2pol((X-medX)/rX, (Y-medY)/rY);
    else
        [angle, vector]=cart2pol((X-medX), (Y-medY));
    end
        
% adjust zero vectors, to avoid divide by zero error
    mark=find(vector==0);
    vector(mark)=vector(mark)+0.01;

% rank vectors and omit upper and lower fractions
if pl < 1.0,
    tb = trimmedwith(vector, pl);% calls new private function 'trimmedwith'
    X2 = X(tb);
    Y2 = Y(tb);
    angle2 = angle(tb);
    vector2 = vector(tb);
end

% turn data inside out for inner envelope 
    inv1=1./vector2;
	[invx invy] = pol2cart(angle2, inv1);
    
% find envelope: the function 'convhull(x,y)' returns indices into the
%   modified [7 Nov 11]
%   x and y vectors of the points on the convex hull
	envelopeO=convhull(X2,Y2);
	% inner convex hull (with the data inside out)
	envelopeI=convhull(invx,invy);
    
% output
envOXY(:,1)=X2(envelopeO);
envOXY(:,2)=Y2(envelopeO);
envIXY(:,1)=X2(envelopeI);
envIXY(:,2)=Y2(envelopeI);

end

% ---------------------------------------------------------------
function [iv] = trimmedwith(x, p)
% returns a vector of indices for trimming vectors of real values
% PT, 8 August 2009 - checked and improved 7 Nov 11
% input
%   x = vector of real values;
%   p = proportion (0.5 - 0.99) of values to be selected
% output
%   iv = vector of 0 (exclude value) and 1 (include value)

plex = (1 - p)/2; % upper and lower excluded fraction, e.g. 0.05 (p = 0.9)
lx = length(x); % e.g. 59
% added 7 Nov 11, to exclude some multiple occurrences of same value
%   when they are at the upper or lower limit
lr = (max(x) - min(x))/1000; % maximum small perturbation
lp = (rand(lx,1) - 0.5)* lr; % random small peturbations about zero
x = x + lp; % peturb the vector
% end of changes 7 Nov 11
upperindex = ceil(lx*(1-plex)); % e.g. 57
lowerindex = ceil(lx*plex); % e.g. 3
sortedx = sort(x); % sort in default ascending order of values
upperx = sortedx(upperindex);% highest included value, e.g. 57th/59
lowerx = sortedx(lowerindex);% lowest included value, e.g. 3rd/59
iv = (x <= upperx) & (x >= lowerx); % i.e., 1 if included, else 0
    % - refers to original vector
end
% ----------------------------------------------------------------